/*  cbtree.h      v 2.00   last mod  6/9/87        */

#ifndef CBTREE_H      /* prevent multiple inclusion */
#define CBTREE_H 1

#include "cbtrdef.h"

    /******************************************************************/
   /*                                                               */
  /* recompile cbtree.c and application pgm if #defines are changed */
 /*                                                                */
/******************************************************************/

#define CBVER 0x0230 /* CBTREE version 230, initiated 10/29/87 */

/* for fixed-length records: */
#define btseek(fd, btloc, reclen)  lseek(fd, ((btloc) - 1L) * (reclen), 0)

/* compatible form for variable length records: reclen can be a dummy */
#define btseekv(fd, btloc, reclen) lseek(fd, (long)(btloc), 0)

#define HDRTRYS   4  /* number of times to try to lock header record */

/* MAXBTREE is NOT a cbtree limitation.  btsetup.c and btrinit.c use this  */
/* value to process the btparms.btr file.  Feel free to increase MAXBTREE. */

#if AmigaDOS
#define MAXBTREE   18   /* up to MAXBTREE B+trees    */
#endif
#if MSDOS
#define MAXBTREE   20
#endif
#if XENIX
#define MAXBTREE   19
#endif

#define MAXFNAME 32 /* Maximun # chars in index file name including PATH.  */
					/* This can be increased as needed. Increasing MAXFNAME*/
					/* will increase the amount of memory required by each */
					/* btcommo structure. */

#define LINWIDTH   82   /* max line length for btparms.btr file */
#define DELIMITER '^'   /* in BTPARMS file */

#ifndef ERR
#define ERR -1
#endif

#ifndef NULL
#define NULL  ((char *)0)      /* define NULL pointer */
#endif

#ifndef NUL
#define NUL  '\0'            /* define NUL character */
#endif

#define repeat   do
#define until(x) while(!(x))
#define uint     unsigned
#define SZCHAR   sizeof(char)

#ifndef MIN
#define MIN(i, j) ((i) < (j) ? i : j)
#endif

#ifndef MAX
#define MAX(i, j) ((i) < (j) ? j : i)
#endif
#define GETREC   1
#define GETNXT   2
#define GETPRV   3
#define GETALL   4
#define INSERT   5
#define DELETE   6
#define DELTKY   7
#define ISRTKY   8
#define GETSEQ   9
#define GETFRST   10
#define GETLAST   11
#define GETPAR   12
#define GETKEYS   13
#define NEWLOC   14
#define GETGT      15
#define GETGE      16
#define GETLT      17
#define GETLE      18

#define BTCALLOK   0
#define BTERROR   -1
#define BTNOTFND   -2
#define BTCOMMO   -3
#define NOMEM      -4
#define BTCPYBRO   -5
#define BTLOCK   -6
#define BTUNLOCK   -7
#define BTABORT   -9

#define CK_NOMEM     1
#define CK_BTRIN     2
#define CK_OPEN   3
#define CK_SEEK   4
#define CK_READ   5
#define CK_WRIT   6
#define CK_CBTR   7
#define CK_STAK   8
#define CK_PUSH   9
#define CK_INDX  10

#define LOCKR    2   	/* Non-blocking lock */
#define UNLOCKR  0     /* Unlock record */

/* function declarations */
#if EC | DC
#else
void  adjstack();
void  ckerror();
void  cberror();
int   cbtree();
int   cmptnode();
int   cpyltbro();
int   cpyrtbro();
int   binarydt();
int   binarysr();
int   btrdelet();
int   btrinit();
int   btcinit();
int   btrisrt();
int   btrgets();
void  creatbtr();
void  deleteky();
int   deletrec();
void  free();
void  free_gtkey();
void  free_svkey();
void  getfhdr();
void  getidxr();
void  insertky();
void  putfhdr();
void  putidxr();
void  putroot();
void  relstack();
void  spltnode();

long  btgivvar();
void  btdelvar();
long  btnxtvar();
uword btsizvar();

/*lint +fvr
  The following functions exhibit variable return modes.
 */

int  lockunlk();
#endif

      /* cbtree structure templates */

/* the following is used by ckerror.c */
#define BTOPTYPE struct _btoptype
BTOPTYPE
{
   char *op_name;
   int   op_value;
};

/* offset to base block for variable length records */
#define BASEBLK   32    /* reserved space at beginning of file */

#define BTBLKHDR struct btblkhdr
struct btblkhdr {
   long freelst;
   long eoflst;
};
#define SZBLKHDR sizeof(BTBLKHDR)

#define BT_BASE struct _bt_base    /* first 28 bytes after BTBLKHDR */
#define SZ_BT_BASE     (BASEBLK-SZBLKHDR)
BT_BASE {
   uword   bb_cbver;      /* CBTREE version number */
   uword   bb_negver;     /* negative CBTREE version number */
   uword   bb_numbtree;   /* # B+tree slots already in this file */
   uword   bb_szinfo;     /* size of each info block */
   uword   bb_blksz;      /* index block size */
   uword   bb_varlen;     /* if variable length records are used */
   char    bb_fill[SZ_BT_BASE-2-2-2-2-2-2];
};

#define BT_INFO     struct _bt_info   /* B+tree parameters */
#define SZ_BT_INFO  32
#define SZ_BNAME     8
BT_INFO
{
   long  bi_rootp;      /* root for this tree */
   char  bi_bname[SZ_BNAME];     /* btree name */
   uword bi_blksz;      /* length of index block parameters */
   uword bi_kylen;      /* key length (w/o NUL) */
   uword bi_cells;      /* cells per index node */
   uword bi_varlen;     /* variable length record flag */
   char  bi_fill[SZ_BT_INFO-SZ_BNAME-4-2-2-2-2];
};

#define BT_LINK     struct _bt_link
BT_LINK
{
   long  bl_link;
   char  bl_tag[SZ_BNAME];   /* make it look like BT_INFO */
   uword bl_blksz;           /* make it look like BT_INFO */
   uword bl_more;           /* # BT_INFO blocks that will fit here */
   uword bl_total;           /* # BT_INFO blocks per index block */
};
#define SZ_BT_LINK  sizeof(BT_LINK)

#define BTIDX struct btidx
struct btidx {
   long   btptr;     /* record location */
   char  *skeynme;   /* key */
};
#define SZBTIDX sizeof(BTIDX)

#define BTIDXBLK struct btidxblk
struct btidxblk {
   uword blkalloc;   /* count of cells in block w/ RAM allocated */
   uword blktype;   /* block type... 1 = data; 0 = idx            */
   uword cellicnt;   /* count of cells in use                    */
   long  fwdpage;   /* ptr to next ordered page; (blktype = 1)   */
   long  bwdpage;   /* ptr to prev. ordered page; (blktype = 1) */
   BTIDX btpage[1];   /* idx node with # of cells + 1 NULL. */
};
#define SZIDXBLK sizeof(BTIDXBLK)

#define BTC struct btcommo
struct btcommo {
   char   *dtaname;      /* Data File Name */
   uword   btdtalen;     /* Data File Record Length */
   uword   btvarlen;     /* variable-length record mode */
   uword   btmulti;      /* muti-user mode (record locking) */
   uword   btcrash;      /* crash-protection mode */
   BTBLKHDR freeptrs;   /* freelst and end-of-file pointers */
   long    findroot;     /* where to find the root block pointer */
   long    btroot;       /* record offset for root block */
   char   *idxname;      /* Index File Name */
   uword   btidxlen;     /* Index File Record Length */
   uword   btkeylen;     /* max strlen(key) */
   uword   btcells;      /* max # Cells per Index Block */
   char   *btname;       /* btree name */
   uword   btoptype;     /* CBTREE operation type */
   char   *btkey;        /* search key for operation */
   long    btloc;        /* data file record location */
   uword   btoptyp2;     /* CBTREE secondary operation */
   char   *btkey2;       /* secondary key */
   long    btloc2;       /* secondary operation */
   uword   btmax;        /* maximum size of btrecnum array */
   long   *btrecnum;     /* array of record numbers     */
   uword   btmax2;       /* maximum size of btrecnm2 array */
   long   *btrecnm2;     /* for logical operations on record ranges */
   char   *btsvkey;      /* save key for use with GETALL, et. al. */
   uword   btsave;       /* save key is in use  */
   long   *locklist;     /* tree traversal list */
   uword   lockptr;      /* locklist index */
   char *(*btmakeky)();  /* function to build the key */
   int   (*btxcmpky)();  /* function to do exact key comparisons */
   int   (*btpcmpky)();  /* function to do partial key comparisons */
};

#endif /*CBTREE_H*/

